%!octave 
%
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.8.0
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: load_data.m,v 1.14 2015/04/29 10:48:49 mechanoid Exp mechanoid $


% каждый класс разделяется в заданной пропорции на три набора
% [ учебный | контрольный | тестовый ]
% номера точек выбираются случайным образом
function retval=load_data(),
   points={};
   cl={};

   r=[ 0.2 0.1 ]; % распределение точек по наборам - учебный | контрольный | тестовый (все остальные)
   points(1)=load('data/1998_2/points_b.txt') ;
   points(2)=load('data/1998_2/points_r.txt') ; 
   points=autoscale(points);
   cl(1)=[ points{1} ] ; 
   cl(2)=[ points{2} ] ;

   % - - - - - - - - - - - - - - - - - - - - 
   % общее количество точек
   sp=size(points,2);s=0; for i=1:sp, s+=size(points{i},1); endfor

   % количество классов
   ncl=size(cl,2) ;

   Xl=Xc=Xt=[] ;
   Yl=Yc=Yt=[] ;

   % случайным образом разделяем каждый класс 
   % на три набора точек в пропорции r
   for i=1:ncl,
	  cli=cl{i} ;
	  sz=size(cli,1) ; % размер класса i
	  % выход сети для класса i
	  out=-ones(sz,ncl) ;
	  out(:,i)*=-1 ;

	  % все номера в случайном порядке 
	  ir=randperm(sz) ;
	  n1=round(sz*r(1)) ; n2=round(sz*r(2))+n1 ;
	  il=ir(1:n1);        % номера точек учебного     набора
	  ic=ir((n1+1):n2) ;  % номера точек контрольного набора
	  it=ir((n2+1):end) ; % номера точек тестового    набора

	  Xl=[ Xl ; cli(il,:) ] ; 
	  Yl=[ Yl ; out(il,:) ] ;
	  
	  Xc=[ Xc ; cli(ic,:) ] ; 
	  Yc=[ Yc ; out(ic,:) ] ;
	  
	  Xt=[ Xt ; cli(it,:) ] ; 
	  Yt=[ Yt ; out(it,:) ] ;
   endfor

   save("-text","result/xl.txt", "Xl" ) ;
   save("-text","result/yl.txt", "Yl" ) ;

   save("-text","result/xc.txt", "Xc" ) ;
   save("-text","result/yc.txt", "Yc" ) ;

   save("-text","result/xt.txt", "Xt" ) ;
   save("-text","result/yt.txt", "Yt" ) ;

   plot_data(Xl,Yl,"xl",-1) ;
   plot_data(Xc,Yc,"xc",-1) ;
   plot_data(Xt,Yt,"xt",-1) ;

   D={ {Xl,Yl}, {Xc,Yc} , {Xt,Yt} } ;

   printf("%i - %i / %i / %i \n",s, size(Xl,1),size(Xc,1),size(Xt,1) );

   retval=D;

endfunction


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=autoscale(P),
   sz=size(P,2);
   mn=[];mx=[];

   for i=1:sz,
	  X=P{i};
	  mn=[mn;min(X)];
	  mx=[mx;max(X)];
   endfor

   mn=min(mn);
   mx=max(mx)-mn;
   sc= 1.0 ./mx; 

   for i=1:sz,
	  X=P{i};
	  X-=mn ;
	  X=X.*sc;
	  P(i)=X;
   endfor

   retval=P;

endfunction

