%!octave 
%
%     MLP back propagation
%      
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.8.0
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: mlp_bp_grad.m,v 1.1 2015/04/30 18:56:25 mechanoid Exp $


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% градиент ошибки сети
function retval=mlp_bp_grad(W,x,y),
   % фаза прямого распространения
   l=mlp_layers( x , W ); % состояния нейронов всех слоёв
   % фаза обратного распространения
   er=mlp_error(W,l,y); % ошибка сети на примере номер i
   g={}; % градиент ошибки сети
   for j=1:size(er,2),
	  g{end+1} = [ mlp_activate(l{j}) -1.0 ]' * ( er{j} ) ;
   endfor
   retval=g;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% состояния нейронов всех слоёв для весов W и входа x
function retval=mlp_layers(x,W),
   cl=size(W,2) + 1 ; % количество слоёв сети, включая вход
   l={}; 
   l{1} = x ;  % вход
   l{2} = [ x -1.0 ] * W{1} ;  % неактивированные нейроны второго слоя
   for j=3:cl, 
	  l(j) = [ mlp_activate(l{j-1})  -1.0 ] * W{j-1} ; 
   endfor
   retval=l;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=mlp_error(W,l,y),
   er={}; % ошибка слоёв
   cl=size(l,2); % количество слоёв, включая вход
   
   % ошибка сети (последнего слоя)
   er{cl-1}=(mlp_activate( l{end} ) - y ) .* mlp_activate_drvt(l{end}) ;
   for j=cl-2:-1:1, % ошибки скрытых слоёв
	  er{j}=er{j+1} * (W{j+1})' .* [ mlp_activate_drvt(l{j+1}) -1.0 ] ; % ошибка слоя j
	  er{j}(end)=[] ; % удаляем ошибку для несуществующей связи (сдвиг)
   endfor
   retval=er;
endfunction

