%!octave 
%
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.8.0
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: mlp_trn_gd.m,v 1.2 2015/06/04 08:44:23 mechanoid Exp $



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% обучение ИНС методом градиентного спуска
%
% L={Xl,Yl} - учебный набор
% С={Xc,Yc} - контрольный набор
% W - начальная матрица весов
%
function [W msqe_hist]=mlp_trn_gd(L,C,W),
   SWE=5; % размер окна контроля за возрастанием ошибки
   MAX_ITER=500 ; % максимальное число циклов обучения
   X=L{1} ; Y=L{2} ; % учебный набор
   szX=size(X,1) ; % количество объектов в учебном наборе
   trsp=2e-1; % скорость обучения (изменения весов)

   % C=L; % не использовать контрольный набор
   
   msqe_hist=[ 100:-1:(100-SWE) ]; % история значений ошибки обучения
   
   for iter=1:MAX_ITER,

      % случайная последовательность учебных примеров
	  for i=randperm(szX),
		 g=mlp_bp_grad(W,X(i,:),Y(i,:)); % градиент ошибки сети
		 p=antigrad(g); % направление изменения параметров
		 %trsp=tr_speed3(W,X(i,:),Y(i,:),p); 
		 W=weight_correct(W,p,trsp); % корректируем веса
	  endfor

 	  % вычисляем ошибку сети на контрольном наборе
	  Yr=mlp_test(C{1},W) ;
	  msqe_hist(end+1)=msqe(C{2},Yr) ;

	  % условия остановки обучения
	  if stop_condition(msqe_hist(end-SWE:end),trsp,iter,MAX_ITER), break ; endif

	  % скорость обучения (изменения весов)
	  % trsp=tr_speed2(trsp,msqe_hist(end-1:end)); 
	  % trsp=tr_speed(iter); 

   endfor

   msqe_hist(1:SWE+1)=[]; % удаляем из истории ошибок начальные значения

   if iter>(MAX_ITER-1) , printf("\nдостигнут порог количества циклов обучения         \n") ; endif

endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% корректируем веса
function retval=weight_correct(W,p,trsp),
   for j=1:length(W),
	  W(j)= W{j} + trsp * p{j} ;
   endfor
   retval=W;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% антиградиент
function retval=antigrad(g), 
   p={}; 
   for j=1:length(g), p{end+1}=-g{j}; endfor 
   retval=p;
endfunction



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% условия остановки обучения
function retval=stop_condition(er,st,iter,max_iter),
   ERR_MIN=8e-2 ; % порог минимальной ошибки обучения
   ERR_MIN_DIFF=1e-7; % порог минимального отклонения ошибки обучения 

   printf("%i/%i : %f / %f   , speed: %f   \r",iter,max_iter,er(end),ERR_MIN, st ) ; 

   if er(end)<ERR_MIN , 
	  printf("\nдостигнут порог минимальной ошибки обучения           \n") ;
	  retval=1;
	  return;
   endif

   if monot_inc(er) ,
	  printf("\nмонотонный рост ошибки обучения           \n") ;
	  retval=1;
	  return;
   endif

   md=max(abs(mean(er)-er)) ; % максимальное отклонение ошибки от среднего значения в окне контроля
   if md < ERR_MIN_DIFF,
	  printf("\nдостигнут порог минимального отклонения ошибки обучения          \n") ;
	  retval=1;
	  return;
   endif

   retval=0;

endfunction 

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=tr_speed(x),
   % t=2.0 ; retval= 1.0/(x*t) ;
   t=7.0 ; retval= 1.0/(x*t) + 1.0/t ;
endfunction


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=tr_speed2(s,er),
   k=1.41;
   ri=1.05;
   rd=0.75;

   if( er(end) > k*er(end-1) ), s*=rd; else s*=ri; endif

   retval=s ;
endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=tr_speed3(W,x,y,p),
   tt=[ 0.0001 0.001 0.01 0.15 ];
   er=[];
   for t=tt, Wt=weight_correct(W,p,t); r=mlp_test(x,Wt) ; er=[ er msqe(y,r) ] ; endfor
   [v i]=min(er);
   retval=tt(i);
endfunction

