%!octave 
%
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.8.0
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 
% $Id: mlp_trn_gd_moment.m,v 1.1 2015/05/05 10:00:46 mechanoid Exp $



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% обучение ИНС методом градиентного спуска с учётом моментов
% (т.е. с учётом предыдущего значения изменения весов)
% 
% L={Xl,Yl} - учебный набор
% С={Xc,Yc} - контрольный набор
% W - начальная матрица весов
%
function [W msqe_hist]=mlp_trn_gd_moment(L,C,W),
   SWE=5; % размер окна контроля за возрастанием ошибки
   MAX_ITER=500 ; % максимальное число циклов обучения
   X=L{1} ; Y=L{2} ; % учебный набор
   szX=size(X,1) ; % количество объектов в учебном наборе
   %trsp=1e-2; % скорость обучения (изменения весов)

   % C=L; % не использовать контрольный набор
   
   msqe_hist=[ 100:-1:(100-SWE) ]; % история значений ошибки обучения
   
   for iter=1:MAX_ITER,

	  trsp=tr_speed(iter); % скорость обучения (изменения весов)

	  % начальные значения изменений весов
	  dW={};for j=1:size(W,2), dW{j}=0.0;endfor
	  for i=randperm(szX), % случайная последовательность учебных примеров
		 g=mlp_bp_grad(W,X(i,:),Y(i,:)); % градиент ошибки сети
		 [W dW]=weight_correct(W,g,trsp,dW); % корректируем веса
	  endfor

 	  % вычисляем ошибку сети на контрольном наборе
	  Yr=mlp_test(C{1},W) ;
	  msqe_hist(end+1)=msqe(C{2},Yr) ;

	  % условия остановки обучения
	  if stop_condition(msqe_hist(end-SWE:end),trsp,iter,MAX_ITER), break ; endif

   endfor

   msqe_hist(1:SWE+1)=[]; % удаляем из истории ошибок начальные значения

   if iter>(MAX_ITER-1) , printf("\nдостигнут порог количества циклов обучения         \n") ; endif

endfunction

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% корректируем веса
function [W dW]=weight_correct(W,g,trsp,dW),
   a=7e-2; % параметр момента
   for j=1:size(W,2),
      d= - trsp * g{j} + a * dW{j} ; % изменение весов с учётом предыдущего значения
      W(j)= W{j} + d ; % корректируем веса
      dW{j}= d ; % сохраняем изменение весов
   endfor
endfunction



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function retval=tr_speed(x),
   TSP=7.0 ; % параметр изменения скорости обучения
   retval= 1.0/(x*TSP) + 1.0/TSP ;
endfunction


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
% условия остановки обучения
function retval=stop_condition(er,st,iter,max_iter),
   ERR_MIN=8e-2 ; % порог минимальной ошибки обучения
   ERR_MIN_DIFF=1e-7; % порог минимального отклонения ошибки обучения 

   printf("%i/%i : %f / %f   , speed: %f   \r",iter,max_iter,er(end),ERR_MIN, st ) ; 

   if er(end)<ERR_MIN , 
	  printf("\nдостигнут порог минимальной ошибки обучения           \n") ;
	  retval=1;
	  return;
   endif

   if monot_inc(er) ,
	  printf("\nмонотонный рост ошибки обучения           \n") ;
	  retval=1;
	  return;
   endif

   md=max(abs(mean(er)-er)) ; % максимальное отклонение ошибки от среднего значения в окне контроля
   if md < ERR_MIN_DIFF,
	  printf("\nдостигнут порог минимального отклонения ошибки обучения          \n") ;
	  retval=1;
	  return;
   endif

   retval=0;

endfunction 


